avalon = avalon or {
    bIsSetup = false,
    bEnableUserCss = false,
    containers = {},
    elements = {},
    alignment = {}
}

function buildUI()
    -- Setup Individual Displays
    if not avalon.bIsSetup then

        avalon.containers.top = Adjustable.Container:new({name = "avalonContainersTop", y="0%", height="10%", autoLoad=false})
        avalon.containers.left = Adjustable.Container:new({name = "avalonContainersLeft", x="0%", y="0%", height="100%", width="20%", autoLoad=false})
        avalon.containers.right = Adjustable.Container:new({name = "avalonContainersRight", x="-20%", y="0%", height="100%", width="20%", autoLoad=false})
        avalon.containers.bottom = Adjustable.Container:new({name = "avalonContainersBottom", y="-10%", height="10%", autoLoad=false})

        avalon.containers.top:attachToBorder("top")
        avalon.containers.left:attachToBorder("left")
        avalon.containers.right:attachToBorder("right")
        avalon.containers.bottom:attachToBorder("bottom")
    
        avalon.containers.top:connectToBorder("left")
        avalon.containers.top:connectToBorder("right")

        avalon.containers.bottom:connectToBorder("left")
        avalon.containers.bottom:connectToBorder("right")

        -- Setup Top Bar
        -- The Top Bar is just a single hbox
        avalon.alignment.topmain = Geyser.HBox:new(
            {
                name = "avalonAlignmentTopMain",
                width = "100%",
                height = "85%"
            },
            avalon.containers.top
        )

        -- Setup Bottom Bar
        -- The Bottom Bar is a vbox that contains a top hbox that holds three gauges and a bottom hbox for whatever content you want
        avalon.alignment.bottommain = Geyser.VBox:new(
            {
                name = "avalonAlignmentBottomMain",
                width = "95%",
                height = "100%"
            },
            avalon.containers.bottom
        )

        avalon.alignment.bottomgauges = Geyser.HBox:new(
            {
                name = "avalonAlignmentBottomgauges",
                width = "100%",
                height = "50%"
            },
            avalon.alignment.bottommain
        )

        avalon.alignment.bottominfo = Geyser.HBox:new(
            {
                name="avalonAlignmentBottominfo",
                width = "100%",
                height = "-50%"
            },
            avalon.alignment.bottommain        
        )

        -- Setup Left Bar
        -- The Left bar consists of a vbox with four internal vboxes for information display
        -- These are named as:
        -- leftmain - Primary vbox that the other vboxes attach to
        -- lefttop - The top most vbox inside leftmain
        -- leftmidtop - The first of two vboxes between the top and bottom. This is the one below lefttop
        -- leftmidbottom - The second of two vboxes between the top and bottom. This is the one above leftbottom and below leftmidtop
        -- leftbottom - The bottom most vbox inside leftmain
        avalon.alignment.leftmain = Geyser.VBox:new(
            {
                name = "avalonAlignmentLeftmain",
                width = "95%",
                height = "100%"
            },
            avalon.containers.left        
        )

        avalon.alignment.lefttop = Geyser.VBox:new(
            {
                name = "avalonAlignmentLefttop",
                width = "100%",
                height = "25%"
            },
            avalon.alignment.leftmain        
        )

        avalon.alignment.leftmidtop = Geyser.VBox:new(
            {
                name = "avalonAlignmentLeftmidtop",
                width = "100%",
                height = "25%"
            },
            avalon.alignment.leftmain        
        )

        avalon.alignment.leftmidbottom = Geyser.VBox:new(
            {
                name = "avalonAlignmentLeftmidbottom",
                width = "100%",
                height = "25%"
            },
            avalon.alignment.leftmain        
        )

        avalon.alignment.leftbottom = Geyser.VBox:new(
            {
                name = "avalonAlignmentLeftbottom",
                width = "100%",
                height = "25%"
            },
            avalon.alignment.leftmain        
        )

        -- Setup Right Bar
        -- The Right Bar consists of vbox that contains a mapper in the top right and chat in the bottom right
        -- The chat utilizes Demonnic's great ECMO https://github.com/demonnic/MDK
        -- rightmain - Primary VB   ox that other VBoxes attach to
        -- rightmap - The Mapper
        -- rightchat - The EMCO Chat window
        avalon.alignment.rightmain = Geyser.VBox:new(
            {
                name = "avalonAlignmentRightmain",
                width = "100%",
                height = "100%"
            },
            avalon.containers.right        
        )
    
        avalon.alignment.rightmap = Geyser.Container:new(
            {
                name = "avalonAlignmentRightmap",
                width = "100%",
                height = "60%"
            },
            avalon.alignment.rightmain      
        )

        avalon.alignment.rightchat = Geyser.Container:new(
            {
                name = "avalonAlignmentRightchat",
                width = "100%",
                height = "40%"
            },
            avalon.alignment.rightmain
        )

        testLabel(avalon.alignment.topmain, "Top Main")
        testLabel(avalon.alignment.bottomgauges, "Bottom Gauges")
        testLabel(avalon.alignment.bottominfo, "Bottom Info")
        testLabel(avalon.alignment.lefttop, "Left Top")
        testLabel(avalon.alignment.leftmidtop, "Left Mid Top")
        testLabel(avalon.alignment.leftmidbottom, "Left Mid Bottom")
        testLabel(avalon.alignment.leftbottom, "Left Bottom")
        testLabel(avalon.alignment.rightmap, "Right Map")
        testLabel(avalon.alignment.rightchat, "Right Chat")

        avalon.bIsSetup = true

    end

end

function testLabel(container, text)

    local newLabel = Geyser.Label:new(
        {
            name = text,
            width = "100%",
            height = "100%",
            fgColor = "black",
            message = "<center>" .. text .. "</center>"
        },
        container
    )
    newLabel:setColor(0,255,0,150)
    table.insert(avalon.elements, newLabel)

end